from .download import download_zip, download_file
from pathlib import Path
down_path = Path(__file__).parent
xz_url = "addons_file"+"/"+down_path.name+"/"
download_zip(xz_url + "Library.zip",down_path)

bl_info = {
    "name": "Frosty Ice Tools",
    "author": "CGB3D MeiU",
    "version": (1, 0, 0),
    "blender": (3, 4, 1),
    "category": "Shaders",
    "location": "View3D > Frosty Ice Tools",
    "description": "",
    "warning": "",
    "wiki_url": "https://blendermarket.com/products/205384/docs/?ref=1353",
    "tracker_url": "https://cgb3d.taobao.com",
    "support": "COMMUNITY",
}

import bpy
from .function_folder.node_ops import NodeOps
from .function_folder.effects_add_remove import Shader_import, Shader_remove, SA_OT_remove_geometry_nodes, ALL_OT_romove_scene, update_asset_category
from .function_folder.Assets_browser_setup import NF_OT_Library_Setup
from .function_folder.mtl_picker import OBJECT_OT_eyedropper_material_picker
from .function_folder.copy_all_mtl import OT_Copymtl
from .function_folder.copy_mtl import SA_OT_copy_materials_1

from .ui_folder.ui import Shader_PT_library_panel

from .ui_folder.frosty_ice_tools_meiu_info import MeiUInfo_Documents, MeiUInfo_Website, MeiUInfo_Panel, MeiUInfo_Twitter, MeiUInfo_Discord, meiu_frosty_ice_tools_load_icons, meiu_frosty_ice_tools_clear_icons



# 用於儲存預覽集合的字典
preview_collections = {}

#註冊列表
classes = [
    MeiUInfo_Documents,
    MeiUInfo_Website,
    MeiUInfo_Twitter,
    MeiUInfo_Discord,
    # MeiUInfo_Panel,
    NodeOps,
    Shader_import,
    Shader_remove,
    SA_OT_remove_geometry_nodes,
    ALL_OT_romove_scene,
    Shader_PT_library_panel,
    SA_OT_copy_materials_1,
    NF_OT_Library_Setup,
    OBJECT_OT_eyedropper_material_picker,
    OT_Copymtl,
]

cls=classes

def register():
    
    from bpy.types import Scene
    from bpy.props import EnumProperty
    from .function_folder.effects_add_remove import meiu_frosty_ice_tools_update_asset_previews, asset_categories


    for cls in classes:
        bpy.utils.register_class(cls)

    meiu_frosty_ice_tools_load_icons()
    bpy.types.Scene.meiu_frosty_ice_tools_shader_ops_called = bpy.props.BoolProperty(default=False)

    
    
    Scene.frosty_ice_tools_my_asset_category = EnumProperty(
        name="Asset Category",
        items=asset_categories,
        description="Choose a category of assets",
        update=update_asset_category,
    )

    
    Scene.frosty_ice_tools_my_asset_previews = EnumProperty(
        name="Asset Previews",
        items=meiu_frosty_ice_tools_update_asset_previews,
        description="Preview of assets",
        
    )


def unregister():
    
    from bpy.types import Scene

    # 反注册所有类，这部分看起来是正确的3D
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)

    meiu_frosty_ice_tools_clear_icons()

    # 在删除自定义属性之前检查它们是否存在B3D
    if hasattr(Scene, 'meiu_frosty_ice_tools_shader_ops_called'):
        del bpy.types.Scene.meiu_frosty_ice_tools_shader_ops_called

    if hasattr(Scene, 'frosty_ice_tools_my_asset_category'):
        del Scene.frosty_ice_tools_my_asset_category

    if hasattr(Scene, 'frosty_ice_tools_my_asset_previews'):
        del Scene.frosty_ice_tools_my_asset_previews


#主程式
if __name__ == "__main__":
    register()

