import bpy, os
from ..function_folder.node_ops import node_ops, insert_shader_between_nodes,add_color_attribute, adjust_node_positions, connect_reroute_to_active_output
from ..function_folder.node_import import node_import, apply_node_group_to_selected_mesh, apply_geometry_nodes_to_selected_mesh

E5G2bE = True


# 資產庫相對路徑
assets_library_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "Library/IMG/All")

# Blender相對路徑，位於插件目錄下 assets資料夾
blender_library_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "Library")

# 设定data.blend文件的路径
library_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "Library/DATA/data.blend")




# 获取资产类别
def get_asset_categories():
    """從資料夾名稱動態生成資產類別清單"""
    categories = [('All', "All", "显示所有资产")]
    if os.path.exists(assets_library_path):
        for name in os.listdir(assets_library_path):
            path = os.path.join(assets_library_path, name)
            if os.path.isdir(path):
                categories.append((name, name, f"分类 {name} 的描述"))

    return categories


# 更新资产类别的回调函数
def update_asset_category(self, context):
    categories = get_asset_categories()
    category_exists = self.frosty_ice_tools_my_asset_category in [cat[0] for cat in categories]
    
    # 如果当前选项不在新的类别列表中，重置到第一个
    if not category_exists:
        self.frosty_ice_tools_my_asset_category = categories[0][0]

    # 如果当前選項的縮略圖不在新的類別列表中，重置到第一个
    # if self.frosty_ice_tools_my_asset_previews not in [cat[0] for cat in meiu_frosty_ice_tools_update_asset_previews(self, context)]:
    self.frosty_ice_tools_my_asset_previews = meiu_frosty_ice_tools_update_asset_previews(self, context)[0][0]
        
    
    print("资产类别已更改或重置。")




asset_categories = get_asset_categories()

# 讀取資產庫中的資產縮略圖
def load_assets(category=None):
    thumbnails = []
    base_path = assets_library_path if category in [None, 'All'] else os.path.join(assets_library_path, category)
    for root, dirs, files in os.walk(base_path):
        for file in files:
            if file.endswith('.png'):
                rel_dir = os.path.relpath(root, assets_library_path)
                rel_file = os.path.join(rel_dir, file)
                thumbnails.append(rel_file if rel_dir != '.' else file)
    


    return thumbnails


# 用於儲存預覽集合的字典
preview_collections = {}

# 更新资产缩略图到UI
def meiu_frosty_ice_tools_update_asset_previews(self, context):
    """更新资产缩略图"""
    enum_items = []
    if context is None:
        return enum_items
    
    # 获取当前选择的资产类别
    category = context.scene.frosty_ice_tools_my_asset_category
    
    # 根据选择的类别读取资产
    thumbnails = load_assets(category)
        
    # 创建一个预览集合
    pcoll = preview_collections.get("main")
    if not pcoll:
        from bpy.utils import previews
        pcoll = previews.new()
        preview_collections["main"] = pcoll

    # 生成枚举项
    if thumbnails:
        
        for i, asset_name in enumerate(thumbnails):
            filepath = os.path.join(assets_library_path, asset_name)
            thumb = pcoll.get(asset_name)
            if thumb is None:
                thumb = pcoll.load(asset_name, filepath, 'IMAGE')
            enum_items.append((asset_name, asset_name, "", thumb.icon_id, i))

    
    
        
    return enum_items



def shader_ops(self, context, shader_name):
    Reroute_1 = "19_sfgw89_Reroute_us64_001"
    Reroute_2 = "19_sfgw89_Reroute_us64_002"
    shader_color_id = "Shaders_Mask_4896"
    
    # obj = bpy.context.active_object
    objs = []
    if bpy.context.selected_objects:
        objs = bpy.context.selected_objects

    material = []
    if objs is not None:
        for obj in objs:
            if obj.type == 'MESH':
                for slot in obj.material_slots:
                    material.append(slot)
    mtl_list_tmp = []

    for mtls in material:
        #get the material in the slot
        mtl = mtls.material
        if mtl is not None:
            mtl_list_tmp.append(mtl)
    
    # 從node_import.py中導入node_import函數
    node_import(shader_name, library_path)

    # 導入之後，將節點組應用到選定的網格
    if shader_name.startswith("Mat_"):
        apply_node_group_to_selected_mesh(shader_name)

    else:
        # 導入幾何節點組到選定的網格
        apply_geometry_nodes_to_selected_mesh(shader_name)

    # 现在，节点组已导入，可以执行node_ops操作
    # node_ops(Reroute_2, "Material Output", "c") 

    connect_reroute_to_active_output()

    # 在Reroute_1和Reroute_2之间插入shader_name
    insert_shader_between_nodes(material, Reroute_1, Reroute_2, shader_name)

    # 添加顏色屬性
    add_color_attribute(shader_color_id, (1, 1, 1, 1),"A")

    #調整節點位置
    for mtl in mtl_list_tmp:
        adjust_node_positions(mtl)


    
    self.report({'INFO'}, f"Material has been added.")


    return {'FINISHED'}


# 導入Shader Node的操作
class Shader_import(bpy.types.Operator):
    """从资产库导入选定的资产到场景中"""
    bl_idname = "frosty_ice_tools_shaders_assetsimpo.rt"
    bl_label = "导入资产"
    #undo
    bl_options = {'REGISTER', 'UNDO'}

    meiu_frosty_ice_tools_shader_ops_called = False  # 类属性，用于跟踪是否已执行 shader_ops

    # 更新这部分以使用动态生成的资产类别
    frosty_ice_tools_my_asset_category: bpy.props.EnumProperty(
        items=lambda self, context: get_asset_categories(),
        name="Shader Category",
        description="选择资产类别来过滤显示的资产",
        default='All',
    ) # type: ignore

    def execute(self, context):
        scn = context.scene

        add_color_attribute("Shaders_Mask_4896", (1, 1, 1, 1),"A")

        #if not object mode
        if context.mode != 'OBJECT':
            self.report({'WARNING'}, "Please switch to Object Mode to run this operation.")
            return {'CANCELLED'}
        asset_name_with_path = scn.frosty_ice_tools_my_asset_previews
        asset_name, _ = os.path.splitext(os.path.basename(asset_name_with_path))
        
        # 检查所选 Mesh 中是否存在 Reroute_1 和 Reroute_2
        reroute_1_name = "19_sfgw89_Reroute_us64_001"
        reroute_2_name = "19_sfgw89_Reroute_us64_002"
        reroute_exists = False
        
        for obj in context.selected_objects:
            
            if obj.type == 'MESH':
                for slot in obj.material_slots:
                    material = slot.material
                    if material and material.use_nodes:
                        nodes = material.node_tree.nodes
                        reroute_1 = nodes.get(reroute_1_name)
                        reroute_2 = nodes.get(reroute_2_name)
                        if reroute_1 and reroute_2:
                            reroute_exists = True
                            break
                if reroute_exists:
                    break
        
        # 设置 meiu_frosty_ice_tools_shader_ops_called 标志
        scn.meiu_frosty_ice_tools_shader_ops_called = reroute_exists

        if scn.meiu_frosty_ice_tools_shader_ops_called and asset_name.startswith("Mat_"):
            
            if E5G2bE:
                self.report({'INFO'}, "着色器操作已成功执行。")
                pass
                #run Shader_remove()
                bpy.ops.frosty_ice_tools_shaders_assets.remove()
                #run Shader_import again
                bpy.ops.frosty_ice_tools_shaders_assetsimpo.rt()
            else:
                self.report({'WARNING'}, "Shader operations can only be executed once. Reroute nodes already exist. Please run the remove operation before running it again.")

            return {'CANCELLED'}

        shader_ops(self, context, asset_name)
        self.report({'INFO'}, "着色器操作已成功执行。")
        return {'FINISHED'}


# 刪除Shader Node的操作
class Shader_remove(bpy.types.Operator):
    """从场景中删除指定的Shader Node，并清理Reroute节点，重新连接节点"""
    bl_idname = "frosty_ice_tools_shaders_assets.remove"
    bl_label = "删除Shader"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        add_color_attribute("Shaders_Mask_4896", (1, 1, 1, 1),"R")
        reroute_1_name = "19_sfgw89_Reroute_us64_001"
        reroute_2_name = "19_sfgw89_Reroute_us64_002"
        shader_color_id = "Shaders_Mask_4896"
        scn = context.scene

        self.meiu_frosty_ice_tools_shader_ops_called = False

        for obj in bpy.context.selected_objects:
            if obj.type != 'MESH':
                continue  # 跳过非Mesh对象

            for slot in obj.material_slots:
                material = slot.material
                if material:
                    nodes = material.node_tree.nodes
                    reroute_1 = nodes.get(reroute_1_name)
                    reroute_2 = nodes.get(reroute_2_name)

                    if not reroute_1 or not reroute_2:
                        print("One of the reroute nodes is missing. Please run the import operation first.")
                        continue  # 如果一个节点缺失，跳过当前材质处理

                    try:
                        reroute_1_input = reroute_1.inputs[0].links[0].from_node if reroute_1.inputs[0].is_linked else None
                        reroute_1_output = reroute_1.outputs[0].links[0].to_node if reroute_1.outputs[0].is_linked else None
                        reroute_2_input_node = reroute_2.inputs[0].links[0].from_node if reroute_2.inputs[0].is_linked else None
                        reroute_2_output = reroute_2.outputs[0].links[0].to_node if reroute_2.outputs[0].is_linked else None
                    except IndexError:
                        print("Some connections are missing.")
                        continue  # 如果某个连接缺失，跳过当前材质处理

                    if reroute_1_output:
                        material.node_tree.nodes.remove(reroute_1_output)
                    if reroute_2_output and reroute_1_input:
                        material.node_tree.links.new(reroute_1_input.outputs[0], reroute_2_output.inputs[0])
                    if reroute_1 and reroute_2:
                        material.node_tree.nodes.remove(reroute_1)
                        material.node_tree.nodes.remove(reroute_2)

                    for node in nodes:
                        if node.get("NodeFX_Tag"):
                            material.node_tree.nodes.remove(node)


        # 清除Blender文件中未被引用的node groups
        unused_node_groups = [ng for ng in bpy.data.node_groups if ng.users == 0]
        for ng in unused_node_groups:
            bpy.data.node_groups.remove(ng)

        self.report({'INFO'}, "All unused Shader Node Groups have been removed.")
        scn.meiu_frosty_ice_tools_shader_ops_called = False
        self.report({'INFO'}, "Shader operations reset. You can now re-run shader operations.")
        return {'FINISHED'}


class SA_OT_remove_geometry_nodes(bpy.types.Operator):
    """从场景中删除所有Geometry Nodes"""
    bl_idname = "blackdeer_node_fx_plus.remove_geometry_nodes"
    bl_label = "删除Geometry Nodes"
    bl_options = {'REGISTER', 'UNDO'}
    

    def execute(self, context):
        scn = context.scene

        for obj in bpy.context.selected_objects:
            if obj.type not in {'MESH', 'CURVE'}:
                continue  # 跳过非Mesh对象

            # 删除 Geometry Nodes 修改器
            geometry_modifiers = [mod for mod in obj.modifiers if mod.type == 'NODES' and mod.name.startswith('Geo_')]
            for mod in geometry_modifiers:
                obj.modifiers.remove(mod)
                print(f"已从 '{obj.name}' 中删除 Geometry Nodes 修改器。")

                
        # 清除Blender文件中未被引用的node groups
        unused_node_groups = [ng for ng in bpy.data.node_groups if ng.users == 0]
        for ng in unused_node_groups:
            bpy.data.node_groups.remove(ng)

        self.report({'INFO'}, "All unused Shader Node Groups have been removed.")
        scn.meiu_frosty_ice_tools_shader_ops_called = False
        self.report({'INFO'}, "Shader operations reset. You can now re-run shader operations.")
        
        return {'FINISHED'}
    
class ALL_OT_romove_scene(bpy.types.Operator):
    """从场景中删除所有Node"""
    bl_idname = "frosty_ice_tools.remove_scene"
    bl_label = "删除所有Node"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        scn = context.scene

        for obj in bpy.context.selected_objects:
            if obj.type not in {'MESH', 'CURVE'}:
                continue
            SA_OT_remove_geometry_nodes.execute(self, context)
            Shader_remove.execute(self, context)

        return {'FINISHED'}