import bpy
# from ..function_folder.modifier_view import draw_geo_modifier_properties

class Shader_PT_library_panel(bpy.types.Panel):
    """在N面板顯示資產庫"""
    bl_label = "Frosty Ice Tools"
    bl_idname = "frosty_ice_tools_my_asset_previews"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Frosty Ice Tools'

    def draw(self, context):
        layout = self.layout

        if context.mode == 'OBJECT':
            obj = context.object

            box = layout.box()

            # 添加资产类别下拉式菜单
            box.prop(context.scene, 'frosty_ice_tools_my_asset_category', text="")
            box.scale_y = 1.2

            layout.separator()
            
            # 繪製資產瀏覽器
            box.template_icon_view(context.scene, "frosty_ice_tools_my_asset_previews", show_labels=False, scale=7.0, scale_popup=6.0)

            # 導入資產按鈕
            box.operator("frosty_ice_tools_shaders_assetsimpo.rt", text="添加", icon='IMPORT')

            # 刪除資產按鈕
            box.operator("frosty_ice_tools.remove_scene", text="移除", icon='X')

        else:
            layout.label(text="Please switch to Object Mode", icon='INFO')


        active_object = context.active_object
        if active_object and 'material_slots' in dir(active_object) and active_object.material_slots:
            # 修正变量名，从 obj 更改为 active_object
            slot = active_object.material_slots[active_object.active_material_index]
            name_endwith = "_N"
            exclude_keyword = "Material Output"
            if slot.material and slot.material.node_tree:
                nodes = [n for n in slot.material.node_tree.nodes if n.name.endswith(name_endwith) and not n.name.startswith(exclude_keyword)]
                if nodes:
                    for node in nodes:
                        box = layout.box()

                        value_inputs = [socket for socket in node.inputs if not socket.hide_value and not socket.is_linked]
                        if value_inputs:
                            for socket in value_inputs:
                                row = box.row()
                                row.scale_x = 1
                                row.scale_y = 1.2
                                socket.draw(context, row, node, socket.name)



        if active_object:
            for modifier in active_object.modifiers:
                if modifier.name.startswith("Geo_"):
                    box = layout.box()
                    col = box.column(align=True)
                    col.context_pointer_set("modifier", modifier)

                    # Header with modifier name and the toggle visibility button
                    row = col.row()
                    row.label(text=modifier.name)
                    row.prop(modifier, "show_viewport", text="")
                    row.prop(modifier, "show_render", text="")
                    row.operator("object.modifier_remove", text="", icon='X').modifier = modifier.name
                    
                    
                    col.separator()

                    # Displaying user-friendly names for Geometry Nodes modifier inputs with support for various socket types
                    if modifier.type == 'NODES' and modifier.node_group:
                        # Get the node group associated with the Nodes modifier
                        node_group = modifier.node_group

                        # Find the Group Input node within the node group
                        group_input_node = next((node for node in node_group.nodes if node.type == 'GROUP_INPUT'), None)

                        if group_input_node:
                            # Iterate over the outputs of the Group Input node
                            for socket in group_input_node.outputs:
                                row = col.row()
                                col.separator()
                                row.scale_y = 1.2
                                friendly_name = socket.name


                                # Handling different socket types
                                if isinstance(socket, bpy.types.NodeSocketFloat):            
                                    row.prop(modifier, f'["{socket.identifier}"]', text=friendly_name)

                                elif isinstance(socket, bpy.types.NodeSocketFloatAngle):  # 首先检查它是否为 NodeSocketFloat 类型
                                    row.prop(modifier, f'["{socket.identifier}"]', text=friendly_name)                    

                                elif isinstance(socket, bpy.types.NodeSocketInt):
                                    row.prop(modifier, f'["{socket.identifier}"]', text=friendly_name)
                                
                                elif isinstance(socket, bpy.types.NodeSocketVector):                               
                                    row.prop(modifier, f'["{socket.identifier}"]', text=friendly_name)
                                
                                elif isinstance(socket, bpy.types.NodeSocketColor):                             
                                    row.prop(modifier, f'["{socket.identifier}"]', text=friendly_name)
                                
                                elif isinstance(socket, bpy.types.NodeSocketObject):
                                    # Create a property selector for an object
                                    row.prop_search(modifier, f'["{socket.identifier}"]', bpy.context.scene, "objects", text=friendly_name)

                                elif isinstance(socket, bpy.types.NodeSocketBool):
                                    row.prop(modifier, f'["{socket.identifier}"]', text=friendly_name)

                                else:
                                    # Default handler for unknown or custom socket types
                                    pass
                        else:
                            print("No Group Input node found in the node group.")

                    # Move up and move down controls
                    move_up = col.operator("object.modifier_move_up", text="Move Up", icon='TRIA_UP')
                    move_up.modifier = modifier.name
                    move_down = col.operator("object.modifier_move_down", text="Move Down", icon='TRIA_DOWN')
                    move_down.modifier = modifier.name

